<!-- 
  This template produces a simplified and consolidated XML containing all the data for an Excel spreadsheet.
  
  In particular, this template:
   - consolidates all worksheets into one file
   - resolves shared strings
   - discards styles
  
  @author Christophe Lauret
  @version 27 October 2011
-->
<xsl:stylesheet version="2.0"
      xmlns:ss="http://schemas.openxmlformats.org/spreadsheetml/2006/main"
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
      xmlns:xs="http://www.w3.org/2001/XMLSchema"
      exclude-result-prefixes="xsl xs ss">

  <xsl:output encoding="UTF-8" method="xml" indent="yes" />

  <!-- folder -->
  <xsl:variable name="folder" select="string-join(tokenize(base-uri(), '/')[not(position() = last())], '/')"/>

  <!-- Load shared strings -->
  <xsl:variable name="sharedstrings" select="document(concat($folder, '/sharedStrings.xml'))"/>

  <!-- Process the workbook -->
  <xsl:template match="ss:workbook">
    <workbook>
      <xsl:for-each select="ss:sheets/ss:sheet"> 
        <xsl:sort select="@sheetId"/>
        <sheet name="{@name}">
          <xsl:apply-templates select="document(concat($folder,'/worksheets/sheet', @sheetId, '.xml'))"/>        
        </sheet>
      </xsl:for-each>
    </workbook>
  </xsl:template>

  <!-- Process a work sheet -->
  <xsl:template match="ss:worksheet">
    <xsl:for-each select="ss:sheetData/ss:row">
      <row r="{@r}" spans="{@spans}">
        <xsl:for-each select="ss:c">
          <col r="{@r}">
            <xsl:choose>
              <!-- Value is a shared string -->
              <xsl:when test="@t = 's'"><xsl:value-of select="ss:get-share-string(ss:v)"/></xsl:when>
              <xsl:otherwise><xsl:value-of select="ss:v"/></xsl:otherwise>
            </xsl:choose>
          </col>
        </xsl:for-each>
      </row>
    </xsl:for-each>
  </xsl:template>

  <xsl:function name="ss:get-share-string">
    <xsl:param name="i"/>
    <xsl:value-of select="$sharedstrings//ss:si[$i + 1]/ss:t"/>
  </xsl:function>

</xsl:stylesheet>

