<?xml version="1.0" encoding="UTF-8"?>

  <!--
    This stylesheet defines the rules to turn PS standard XML into FO XML format.
    TODO: check bookmark rules at the bottom of this stylesheet.
    
    @author Jean-Baptiste Reure
    @author Philip Rutherford
    @author Willy Ekasalim
    
    @version 3 May 2012
    
    Copyright (C) 2012 Weborganic Systems Pty. Ltd.
  -->
  
<xsl:stylesheet version="2.0" 
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:ps="http://www.pageseeder.com/editing/2.0"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:psf="http://www.pageseeder.com/function"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                exclude-result-prefixes="ps psf xs">


  <!-- XRef utilities functions-->
  <xsl:import href="../../Common/XRefUtilities.xsl" />
  <!-- FO utility functions -->
  <xsl:import href="../../Common/FOUtilities.xsl" />
  <!-- Page functions -->
  <xsl:import href="../../Layout/Common/page-functions.xsl"/>
  
  <xsl:output method="xml" version="1.0" indent="no"/>

  <!-- File path -->
  <xsl:param name="filefolder" />
  <!-- Document folder -->
  <xsl:param name="documentfolder" />
  <!-- File name -->
  <xsl:param name="filename" />
  <!-- Style owner -->
  <xsl:param name="styleowner" select="''" />
  <!-- Style format -->
  <xsl:param name="styleformat" select="''" />
  <!-- Style owner -->
  <xsl:param name="styleconfig" select="''" />
  <!-- Cutsom folder -->
  <xsl:param name="customfolder" select="''" />
  <!-- Host URL  -->
  <xsl:param name="hosturl" />
  <!-- URI id -->
  <xsl:param name="uriid" />
  <!-- TOC display -->
  <xsl:param name="xsl-toc" select="''" />
  <!-- Document title -->
  <xsl:param name="uriusertitle" select="''"/>
  <!-- Site prefix -->
  <xsl:param name="siteprefix" select="''"/>
  <!-- Style group name  -->
  <xsl:param name="stylegroupname" select="''"/>
  <!-- jsessionid, used to retrieve images with URI ID href  -->
  <xsl:param name="jsessionid" select="''"/>  
  
<!-- ================================ Table ============================================= -->

<!-- Template for PageSeeder Table element

.. admonition:: xpath:table

   | <fo:block>
   |   <fo:table table-layout="fixed" border-collapse="collapse" inline-progression-dimension.optimum="100%">
   |     <fo:table-column column-width=""/>
   |     <fo:table-body>
   |       ...
   |     </fo:table-body>
   |   </fo:table>
   | </fo:block>

-->
  <xsl:template match="table">
    <fo:block>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:if test="@role">
        <xsl:sequence select="psf:style-properties-role(., 'table', @role)" />
      </xsl:if>
      <!--  currently, fop 0.95 does not support caption -->
      <!-- 
      <fo:table-and-caption>
      <xsl:if test="caption">
	      <fo:table-caption>
	        <fo:block>
	            <xsl:value-of select="caption"/>
	        </fo:block>
	      </fo:table-caption>
      </xsl:if-->
      <fo:table border-collapse="collapse" table-layout="fixed" inline-progression-dimension.optimum="100%">
    		<xsl:copy-of select="@width"/>
    		<xsl:copy-of select="@height"/>
        <xsl:for-each select="row">
          <xsl:sort select="count(cell | hcell)" data-type="number" order="descending"/>
          <xsl:if test="position()=1">
            <xsl:variable name="columns" select="../tgroup"/>
            <xsl:choose>
              <xsl:when test="$columns">
                <xsl:for-each select="$columns/colspec">
                  <fo:table-column column-width="proportional-column-width({@colwidth})"/>
                </xsl:for-each>
              </xsl:when>
              <xsl:otherwise>
                <xsl:for-each select="cell | hcell">
                  <xsl:variable name="element-name" select="concat('table-', local-name())"/>
                  <xsl:variable name="width-style-property" select="psf:load-style-properties-more(psf:load-config(.), $element-name, 'property', string(@role))[@name='width']" />
                  <xsl:variable name="width">
                    <xsl:choose>
                      <xsl:when test="@width"><xsl:value-of select="@width" /></xsl:when>
                      <xsl:when test="$width-style-property"><xsl:value-of select="$width-style-property/@value" /></xsl:when>
                      <xsl:otherwise>1</xsl:otherwise>
                    </xsl:choose>
                  </xsl:variable>
                  <fo:table-column>
                    <xsl:attribute name="column-width">
                      <xsl:text>proportional-column-width(</xsl:text>
                      <xsl:value-of select="replace($width, '\D', '')"/>
                      <xsl:text>)</xsl:text>
                    </xsl:attribute>
                  </fo:table-column>
                </xsl:for-each>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:if>
        </xsl:for-each>
        <fo:table-body>
          <xsl:apply-templates />
        </fo:table-body>
      </fo:table>
      <!-- </fo:table-and-caption> -->
    </fo:block>
  </xsl:template>

  <xsl:template match="caption" />
   
<!-- Template for PageSeeder Row element

.. admonition:: xpath:row

   | <fo:table-row>
   |   <fo:table-cell space-before.optimum="5pt" space-after.optimum="5pt">
   |     <fo:block text-align="left" margin-left="0.08cm" space-before="0.08cm" margin-right="0.08cm">
   |       ...
   |     </fo:block>
   |   </fo:table-cell>
   | </fo:table-row>

-->  
  <xsl:template match="row">
    <fo:table-row>
      <xsl:for-each select="cell | hcell">
        <!-- adding in for colspan support -->
        <fo:table-cell space-before.optimum="5pt" space-after.optimum="5pt">
          <xsl:sequence select="psf:style-properties(., concat('table-', name()))" />
          <xsl:if test="@role">
            <xsl:sequence select="psf:style-properties-role(., concat('table-', name()), @role)" />
          </xsl:if>
          <xsl:if test="@colspan">
            <xsl:attribute name="number-columns-spanned"><xsl:value-of select="@colspan"/></xsl:attribute>
          </xsl:if>
			    <xsl:if test="@valign">
            <xsl:attribute name="display-align">
              <xsl:choose>
                <xsl:when test="@valign='top'">before</xsl:when>
                <xsl:when test="@valign='middle'">center</xsl:when>
                <xsl:when test="@valign='bottom' or @valign='baseline'">after</xsl:when>
              </xsl:choose>
    				</xsl:attribute>
    			</xsl:if>
  			  <!-- adding in for rowspan support -->
          <xsl:if test="@rowspan">
            <xsl:attribute name="number-rows-spanned">
              <xsl:value-of select="@rowspan"/>
            </xsl:attribute>
          </xsl:if>
          <!-- add border support -->
          <xsl:if test="ancestor::table[@border]">
            <xsl:attribute name="border-style">solid</xsl:attribute>
            <xsl:attribute name="border-color">black</xsl:attribute>
            <xsl:attribute name="border-width"><xsl:value-of select="(ancestor::table/@border)[1]"/>px</xsl:attribute>
          </xsl:if>
          <xsl:variable name="align">
            <xsl:choose>
              <xsl:when test="@align">
                <xsl:choose>
                  <xsl:when test="@align='center'">center</xsl:when>
                  <xsl:when test="@align='right'">end</xsl:when>
                  <xsl:when test="@align='justify'">justify</xsl:when>
                  <xsl:otherwise>start</xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:otherwise>start</xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <fo:block text-align="{$align}" margin-left="0.08cm" space-before="0.08cm" margin-right="0.08cm">
            <xsl:attribute name="font-weight">bold</xsl:attribute>
            <xsl:sequence select="psf:style-properties(., name(.))"/>
            <xsl:if test="@role">
              <xsl:sequence select="psf:style-properties(., concat(name(),'-role-', @role))"/>
            </xsl:if>
            <xsl:apply-templates />
          </fo:block>
        </fo:table-cell>
      </xsl:for-each>
    </fo:table-row>
  </xsl:template>
  
<!-- ============================== Body ================================== -->
<!-- Template for PageSeeder Body

.. admonition:: xpath:body

   | <fo:block id="1" break-before="page">
   |   ...
   | </fo:block>

-->
  <xsl:template match="body[../title/@level='table' or empty(*)]" />
    
  <xsl:template match="body[not(../title/@level='table')][*]">
    <xsl:apply-templates select="preceding-sibling::*[1][self::title]" />
    <!-- If there is a section table (e.g. Task) -->
    <fo:block>
      <xsl:if test="not(preceding::body[@id=current()/@id])">
        <xsl:attribute name="id" select="@id" />
      </xsl:if>
      <xsl:if test="string(@type) != ''">
        <xsl:sequence select="psf:style-properties(., concat('body-', @type))"/>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="@pageBreak='before'">
          <xsl:attribute name="break-before">page</xsl:attribute>
        </xsl:when>
        <xsl:when test="@pageBreak='after'">
            <xsl:attribute name="break-after">page</xsl:attribute>
        </xsl:when>
      </xsl:choose>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

<!-- Template for PageSeeder Section (Table of Content)

.. admonition:: xpath:section[toc]

   | <fo:block id="TOC">
   |   <fo:block>Contents</fo:block>
   |   <fo:block>
   |     <fo:block text-align-last="justify">
   |       <fo:basic-link internal-destination="doc">...</fo:basic-link>
   |       <fo:leader leader-pattern="dots"/>
   |       <fo:page-number-citation ref-id="TOC" />
   |     </fo:block>
   |   </fo:block>
   | </fo:block>

-->  
  <xsl:template match="toc[$xsl-toc = 'true']">
    <xsl:variable name="toc" select="." />
    <!-- only include the headings that are in this document (for tocs of sub-documents) -->
    <xsl:variable name="my-headings" select="concat('|', string-join(parent::section/parent::root//*[starts-with(@id, 'toc-')]/@id, '|'), '|')" />
    <xsl:variable name="levels" select="$titles/*[@idref][contains($my-headings, concat('|', @idref, '|'))]" />
    <xsl:variable name="hide" select="psf:load-style-properties(., 'toc')[@name = 'ps-hideIfNotFirst']/@value"/>
    <xsl:if test="count($levels) gt 0 and (empty(ancestor::root/ancestor::root) or $hide = 'false')">
      <fo:block id="{../@id}">
        <xsl:sequence select="psf:style-properties(., 'toc')"/>
        <fo:block>
          <xsl:sequence select="psf:style-properties(., 'toc-title')"/>
          <xsl:text>Contents</xsl:text>
        </fo:block>
        <fo:block>
          <xsl:variable name="max-level" select="number(@maxLevel)" />
          <xsl:for-each select="$levels">
            <xsl:variable name="level" select="number(substring-after(name(), 'ps:level'))"/>
            <xsl:variable name="hide" select="psf:load-style-properties($toc, name())[@name = 'ps-hide']/@value"/>
            <xsl:if test="(not($max-level) or $level le $max-level) and not($hide = 'true') and $level gt 0">
              <xsl:variable name="id" select="@idref" />
              <fo:block>
                <xsl:sequence select="psf:style-properties($toc, name())"/>
                <fo:basic-link>
                  <xsl:attribute name="internal-destination"><xsl:value-of select="$id"/></xsl:attribute>
                  <xsl:value-of select="@label" />
                  <xsl:apply-templates/>
                </fo:basic-link>
                <!-- Add a minimum leader length to make sure it start a new line when appropriate -->
                <fo:leader leader-length.minimum="0.5cm" leader-pattern="dots"/>
                <fo:page-number-citation keep-with-previous="always" ref-id="{$id}" />
              </fo:block>
            </xsl:if>
          </xsl:for-each>
        </fo:block>
      </fo:block>
    </xsl:if>
  </xsl:template>

  <xsl:template name="level">
    <xsl:param name="title" />
    <xsl:param name="level" select="1" />
    <xsl:choose><xsl:when test="contains($title, '.')">
      <xsl:call-template name="level">
        <xsl:with-param name="level" select="$level+1" />
        <xsl:with-param name="title" select="substring-after($title, '.')" />
      </xsl:call-template>
    </xsl:when><xsl:otherwise>
      <xsl:value-of select="$level" />
    </xsl:otherwise></xsl:choose>
  </xsl:template>
  
  <xsl:template name="indent">
    <xsl:param name="title" />
    <xsl:if test="contains($title, '.')">
	  <xsl:text>&#160;&#160;&#160;&#160;</xsl:text>
      <xsl:call-template name="indent">
        <xsl:with-param name="title" select="substring-after($title, '.')" />
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

<!-- ============================== Heading ================================= -->
<!-- Template for PageSeeder Heading 1

.. admonition:: xpath:heading1

   | <fo:block id="1">
   |   1. ...
   | </fo:block>

--> 
  <xsl:template match="heading1">
    <fo:block id="{@id}">
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
        <xsl:if test="@numbered = 'true'">
			<xsl:call-template name="display-title">
	  			<xsl:with-param name="elem" select="." />
		  	</xsl:call-template>
	  	</xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

<!-- Template for PageSeeder Heading 2

.. admonition:: xpath:heading2

   | <fo:block id="1.1">
   |   1.1 ...
   | </fo:block>

--> 
  <xsl:template match="heading2">
    <fo:block id="{@id}">
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:if test="@numbered = 'true'">
			  <xsl:call-template name="display-title">
	  			<xsl:with-param name="elem" select="." />
		  	</xsl:call-template>
	  	</xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

<!-- Template for PageSeeder Heading 3

.. admonition:: xpath:heading3

   | <fo:block id="1.1.1">
   |   1.1.1 ...
   | </fo:block>

--> 
  <xsl:template match="heading3">
    <fo:block id="{@id}">
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:if test="@numbered = 'true'">
        <xsl:call-template name="display-title">
	  			<xsl:with-param name="elem" select="." />
		  	</xsl:call-template>
	  	</xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

<!-- Template for PageSeeder Heading 4

.. admonition:: xpath:heading4

   | <fo:block id="1.1.1.1">
   |   1.1.1.1 ...
   | </fo:block>

--> 
  <xsl:template match="heading4">
    <fo:block id="{@id}">
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:if test="@numbered = 'true'">
        <xsl:call-template name="display-title">
	  			<xsl:with-param name="elem" select="." />
		  	</xsl:call-template>
	  	</xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

<!-- Template for PageSeeder Heading 5

.. admonition:: xpath:heading5

   | <fo:block id="1.1.1.1.1">
   |   1.1.1.1.1 ...
   | </fo:block>

--> 
  <xsl:template match="heading5">
    <fo:block id="{@id}">
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:if test="@numbered = 'true'">
        <xsl:call-template name="display-title">
	  			<xsl:with-param name="elem" select="." />
		  	</xsl:call-template>
	  	</xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

<!-- Template for PageSeeder Heading 6

.. admonition:: xpath:heading6

   | <fo:block id="1.1.1.1.1.1">
   |   1.1.1.1.1.1 ...
   | </fo:block>

--> 
  <xsl:template match="heading6">
    <fo:block id="{@id}">
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:if test="@numbered = 'true'">
        <xsl:call-template name="display-title">
	  			<xsl:with-param name="elem" select="." />
		  	</xsl:call-template>
	  	</xsl:if>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>
  
<!-- Template for PageSeeder Heading (when level > 6)

.. admonition:: xpath:heading

   | <fo:block>
   |   ...
   | </fo:block>

--> 
  <xsl:template match="heading">
    <fo:block>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

<!-- ============================== Para ================================== -->

<!-- Template for PageSeeder Para

.. admonition:: xpath:para

   | <fo:block>
   |  ...
   | </fo:block>

-->
  <!-- pslabelvalues para -->
  <xsl:template match="para[parent::body/parent::section/@format='pslabelvalues']">
    <xsl:if test="position() = 1">
      <xsl:variable name="display_table">
        <!-- Convert section with format='pslabelvalues' into a table -->
        <table role="pslabelvalues">
          <xsl:for-each select=". | following-sibling::para">
            <row>
              <hcell role="pslabelvalues"><xsl:value-of select="text()"/></hcell>
              <cell role="pslabelvalues"><xsl:copy-of select="inlineLabel/node()"/></cell>
            </row>
          </xsl:for-each>
        </table>
      </xsl:variable>
      <xsl:apply-templates select="$display_table/table"/>
    </xsl:if>
  </xsl:template>
  <!-- normal para -->
  <xsl:template match="para[not(parent::body/parent::section/@format='pslabelvalues')]">
    <xsl:variable name="properties" select="psf:load-style-properties(., local-name(.))" />
    <xsl:choose>
      <xsl:when test="@indent and @label">
        <xsl:variable name="ps-indent-px" select="$properties[@name = 'ps-indent-px']/@value" />
        <fo:table table-layout="fixed" border-style="none">
           <fo:table-column column-width="{number(@indent) * number($ps-indent-px)}px"/>
           <fo:table-column />
           <fo:table-body>
            <fo:table-row>
             <fo:table-cell text-align="right" padding-right="5px">
              <fo:block><xsl:value-of select="@label" /></fo:block>
             </fo:table-cell>
             <fo:table-cell>
              <fo:block>
                <xsl:for-each select="$properties[@name != 'ps-indent-px'][@name != 'start-indent']">
                  <xsl:attribute name="{@name}"><xsl:value-of select="@value" /></xsl:attribute>
                </xsl:for-each>
                <xsl:apply-templates select="* | text()" />
              </fo:block>
             </fo:table-cell>
            </fo:table-row>
           </fo:table-body>
          </fo:table>
      </xsl:when>
      <xsl:otherwise>
        <fo:block>
          <xsl:if test="@indent">
            <xsl:variable name="ps-indent-px" select="$properties[@name = 'ps-indent-px']/@value" />
            <xsl:attribute name="start-indent"><xsl:value-of select="number(@indent) * number($ps-indent-px)" />px</xsl:attribute>
            <xsl:if test="@label">
              <xsl:attribute name="text-indent">-<xsl:value-of select="number(@indent) * number($ps-indent-px)" />px</xsl:attribute>
            </xsl:if>
          </xsl:if>
          <xsl:for-each select="$properties[@name != 'ps-indent-px'][string(current()/@indent) = '' or @name != 'start-indent']">
            <xsl:attribute name="{@name}"><xsl:value-of select="@value" /></xsl:attribute>
          </xsl:for-each>
          <xsl:if test="@indent and @label">
            <xsl:variable name="ps-indent-px" select="$properties[@name = 'ps-indent-px']/@value" />
            <fo:inline width="{number(@indent) * number($ps-indent-px)}" border-width="thin" border-style="solid"><xsl:value-of select="@label" /></fo:inline>
          </xsl:if>
          <xsl:apply-templates select="* | text()" />
        </fo:block>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

<!-- ============================== ParaLabel ================================== -->

<!-- Template for PageSeeder ParaLabel

.. admonition:: xpath:paraLabel

   | <fo:block>
   |   <fo:block>
   |   ... the paraLabel name
   |   </fo:block>
   |  ...
   | </fo:block>

-->
  <xsl:template match="paraLabel">
    <xsl:variable name="name" select="concat('-',@name)"/>
    <fo:block>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:sequence select="psf:style-properties(., concat(local-name(.), $name))"/>
      <!-- check if we should show the name of the label -->
      <xsl:variable name="show" select="not(psf:load-style-properties(., 'paraLabelName')[@name = 'ps-hide']/@value = 'true') and 
                          not(psf:load-style-properties(., concat('paraLabelName',$name))[@name = 'ps-hide']/@value = 'true')"/>
      <xsl:if test="$show">
        <fo:block>
          <xsl:sequence select="psf:style-properties(., concat(local-name(.),'Name'))"/>
          <xsl:sequence select="psf:style-properties(., concat(local-name(.),'Name',$name))"/>
          <xsl:value-of select="@name"/>
        </fo:block>
      </xsl:if>
      <xsl:apply-templates select="* | text()" />
    </fo:block>
  </xsl:template>

<!-- Template for PageSeeder InlineLabel

.. admonition:: xpath:inlineLabel

   | <fo:inline>
   |   <fo:inline>
   |   ... the inlineLabel name
   |   </fo:inline>
   |  ...
   | </fo:inline>

-->
  <xsl:template match="inlineLabel">
    <xsl:variable name="name" select="concat('-',@name)"/>
    <xsl:variable name="show" select="not(psf:load-style-properties(., 'inlineLabel')[@name = 'ps-hide']/@value = 'true') and 
                        not(psf:load-style-properties(., concat('inlineLabel',$name))[@name = 'ps-hide']/@value = 'true')"/>
    <xsl:if test="$show">
      <fo:inline>
        <xsl:sequence select="psf:style-properties(., local-name(.))" />
        <xsl:sequence select="psf:style-properties(., concat(local-name(.),$name))"/>
        <xsl:variable name="showName" select="not(psf:load-style-properties(., 'inlineLabelName')[@name = 'ps-hide']/@value = 'true') and 
                                not(psf:load-style-properties(., concat('inlineLabelName',$name))[@name = 'ps-hide']/@value = 'true')"/>
        <xsl:if test="$showName">
  	      <fo:inline>
            <xsl:sequence select="psf:style-properties(., concat(local-name(.),'Name'))"/>
            <xsl:sequence select="psf:style-properties(., concat(local-name(.),'Name',$name))"/>
  	        <xsl:value-of select="@name"/>
  	      </fo:inline>
        </xsl:if>
        <xsl:apply-templates select="* | text()" />
      </fo:inline>
    </xsl:if>
  </xsl:template>

<!-- ============================== Code ================================== -->  
<!-- Template for PageSeeder code 

.. admonition:: xpath:code

   | <fo:block>
   |   ...
   | </fo:block>

-->
  <xsl:template match="code">
  	<fo:block>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
  	  <xsl:apply-templates select="* | text()" />
  	</fo:block>
  </xsl:template>

<!-- ============================== Code ================================== -->  
<!-- Template for PageSeeder monospace 

.. admonition:: xpath:monospace

   | <fo:inline>
   |   ...
   | </fo:inline>

-->
  <xsl:template match="monospace">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:apply-templates select="* | text()" />
    </fo:inline>
  </xsl:template>
  
<!-- ============================== Subscript ================================== -->

<!-- Template for PageSeeder subscript (sub)

.. admonition:: xpath:sub

   | <fo:inline>
   |   ...
   | </fo:inline>

-->  
  <xsl:template match="sub">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:apply-templates select="* | text()" />
    </fo:inline>
  </xsl:template>

<!-- ============================== Superscript ================================== -->
<!-- Template for PageSeeder superscript (sup)

.. admonition:: xpath:sup

   | <fo:inline>
   |   ...
   | </fo:inline>

-->  
  <xsl:template match="sup">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:apply-templates select="* | text()" />
    </fo:inline>
  </xsl:template>
  
  <xsl:template name="display-title">
  	<xsl:param name="elem" />
	<xsl:value-of select="@label"/>
  </xsl:template>


<!-- ============================== Link ================================== -->
<!-- Template for PageSeeder link

.. admonition:: xpath:link[@href]

   | <fo:inline>
   |   <fo:base-link internal-destination=""> ... </fo:base-link>
   | </fo:inline>

-->  
  <xsl:template match="link[@href]">
    <fo:inline>
      <xsl:element name="fo:basic-link">
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
        <xsl:choose>
          <xsl:when test="starts-with(@href,'#')">
            <xsl:attribute name="internal-destination">
              <xsl:value-of select="if (string-length(@href) = 1) then 'first-page' else substring-after(@href,'#')"/>
            </xsl:attribute>      
          </xsl:when>
          <xsl:when test="@href=''">
            <xsl:attribute name="internal-destination">first-page</xsl:attribute>      
          </xsl:when>
          <xsl:otherwise>
            <xsl:attribute name="external-destination">
              <xsl:value-of select="@href"/>
            </xsl:attribute>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:apply-templates/>
      </xsl:element>
    </fo:inline>
  </xsl:template>

<!-- ============================== List | nlist ================================== -->
<!-- Template for PageSeeder list or nlist

.. admonition:: xpath:list|nlist

   | <fo:list-block>
   |   ...
   | </fo:list-block>

-->
  <xsl:template match="list | nlist">
	  <fo:list-block>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
		  <xsl:apply-templates/>
	  </fo:list-block>
  </xsl:template>



<!-- Template for PageSeeder list or nlist.

.. admonition:: xpath:item

   | <fo:list-item>
   |   <fo:list-item-label>
   |     <fo:block> ... </fo:block>
   |     </fo:list-item-label>
   |     <fo:list-item-body>
   |     <fo:block> ... </fo:block>
   |   </fo:list-item-body>
   | </fo:list-item>

-->
  <xsl:template match="item">
    <fo:list-item>
      <xsl:sequence select="psf:style-properties(., 'list-item')"/>
      <xsl:variable name="color-style-property" select="psf:load-style-properties(., 'list-item')[@name='color']" />
      <xsl:variable name="color" select="if ($color-style-property) then $color-style-property/@value else '#1f4f76'" />
  		<fo:list-item-label end-indent="label-end()">
  		  <fo:block>
  		  	<xsl:variable name="level" select="count(ancestor::nlist)"/>
          <xsl:variable name="pos" select="count(preceding-sibling::item) + 1"/>
          <xsl:variable name="label" select="if (../@start) then $pos + xs:integer(../@start) - 1 else $pos"/>
  		  	<xsl:choose>
            <xsl:when test="parent::list">
              <xsl:choose>
                <xsl:when test="../@type='none'"></xsl:when>
                <xsl:when test="../@type='disc'"><fo:inline font-family="Symbol" color="{$color}">&#x2022;</fo:inline></xsl:when>
                <xsl:when test="../@type='circle'"><fo:inline font-size="7pt" font-family="ZapfDingbats" color="{$color}">&#x274D;</fo:inline></xsl:when>
                <xsl:when test="../@type='square'">
                  <xsl:attribute name="margin-top">-1pt</xsl:attribute>
                  <fo:inline font-size="6pt" font-family="ZapfDingbats" color="{$color}" >&#x25A0;</fo:inline>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:attribute name="margin-top">-3pt</xsl:attribute>
                  <fo:inline font-family="Symbol" color="{$color}">&#x2022;</fo:inline>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:when test="parent::nlist/@type = 'arabic'"><xsl:number value="$label" format="1."/></xsl:when>
            <xsl:when test="parent::nlist/@type = 'upperalpha'"><xsl:number value="$label" format="A."/></xsl:when>
            <xsl:when test="parent::nlist/@type = 'loweralpha'"><xsl:number value="$label" format="a."/></xsl:when>
            <xsl:when test="parent::nlist/@type = 'upperroman'"><xsl:number value="$label" format="I."/></xsl:when>
            <xsl:when test="parent::nlist/@type = 'lowerroman'"><xsl:number value="$label" format="i."/></xsl:when>
  				  <xsl:when test="$level=2"><xsl:number value="$label" format="a."/></xsl:when>
  				  <xsl:when test="$level=3"><xsl:number value="$label" format="i."/></xsl:when>
  				  <xsl:otherwise><xsl:number value="$label" format="1."/></xsl:otherwise>
  			 </xsl:choose>
  		  </fo:block>
  		</fo:list-item-label>
  		<fo:list-item-body start-indent="body-start()">
  		  <fo:block>
  			<xsl:apply-templates/>
  		  </fo:block>
  		</fo:list-item-body>
	  </fo:list-item>
  </xsl:template>

<!-- ============================== Title ================================== -->
<!-- Template for PageSeeder list or nlist.  nitem include for backward compatibility only

.. admonition:: xpath:title

   | <fo:block> 
   |   ...
   | </fo:block>

-->  
  <xsl:template match="title">
    <xsl:choose>
      <!-- only show section title if there is actual content -->
      <xsl:when test="parent::section">
        <fo:block space-before.optimum="15pt">
          <xsl:choose>
            <xsl:when test="@level='2' and ../body/node()">
              <fo:block>
                <xsl:sequence select="psf:style-properties(., 'section-title2')" />
                <xsl:value-of select="." />
              </fo:block>
            </xsl:when>
            <xsl:when test="../node()">
              <fo:block>
                <xsl:sequence select="psf:style-properties(., 'section-title')" />
                <xsl:value-of select="." />
              </fo:block>
            </xsl:when>
          </xsl:choose>
        </fo:block>
      </xsl:when>
      <!-- body title -->
      <xsl:otherwise>
        <fo:block>
          <xsl:sequence select="psf:style-properties(., 'title')"/>
          <xsl:value-of select="." />
        </fo:block>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

<!-- ============================== Bold ================================== -->
<!-- Template for PageSeeder bold

.. admonition:: xpath:bold

   | <fo:inline font-weight="bold">
   |   ...
   | </fo:inline>

-->  
  <xsl:template match="bold">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:apply-templates select="* | text()" />
    </fo:inline>
  </xsl:template>

<!-- ============================== Italic ================================== -->
<!-- Template for PageSeeder italics

.. admonition:: xpath:italic

   | <fo:inline font-weight="italic">
   |   ...
   | </fo:inline>

--> 
  <xsl:template match="italic">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:apply-templates select="* | text()" />
    </fo:inline>
  </xsl:template>

<!-- ============================== Underline ================================== -->
<!-- Template for PageSeeder underline

.. admonition:: xpath:underline

   | <fo:inline font-weight="underline">
   |   ...
   | </fo:inline>

--> 
  <xsl:template match="underline">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:apply-templates select="* | text()" />
    </fo:inline>
  </xsl:template>

<!-- ============================== Xref ================================== -->
<!-- Template for PageSeeder xref

.. admonition:: xpath:xref

   | <fo:inline>
   |   <fo:basicc-link internal-destination="">
   |     ...
   |   </fo:basic-link>
   | </fo:inline>

-->
  <xsl:template match="xref">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>
      <xsl:call-template name="xref-link"/>
    </fo:inline>
  </xsl:template>

<!-- Template for PageSeeder blockXref

.. admonition:: xpath:blockXref

   | <fo:inline>
   |   <fo:basic-link internal-destination="">
   |     ...
   |   </fo:basic-link>
   | </fo:inline>

-->
  <xsl:template match="blockXref">
    <xsl:choose>
      <xsl:when test="@Type = 'Transclude'">
        <xsl:apply-templates />
      </xsl:when>
      <xsl:when test="@Type = 'Embed'">
        <xsl:apply-templates />
      </xsl:when>
      <xsl:otherwise><!-- output like a normal XRef? -->
        <fo:block>
          <xsl:sequence select="psf:style-properties(., local-name(.))"/>
          <xsl:call-template name="xref-link"/>
        </fo:block>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
<!-- Template for PageSeeder xref or blockXref link -->
  <xsl:template name="xref-link">
    <xsl:element name="fo:basic-link">
      <xsl:choose>
        <xsl:when test="starts-with(@href,'#psFrag')">
          <xsl:attribute name="internal-destination">
            <xsl:value-of select="if (contains(@href, 'Frag')) then substring-after(@href, 'Frag') else 'first-page'" />
          </xsl:attribute>
        </xsl:when>
        <!-- added by JB on 29/07/2010 -->
        <!-- internal link, could be from 'replaced' section -->
        <xsl:when test="@uriid = $uriid">
          <xsl:attribute name="internal-destination">
            <xsl:value-of select="if (@frag='default') then concat($uriid, '-default') else @frag" />
          </xsl:attribute>
        </xsl:when>
        <!-- addition wliem 24/7/2007 -->
        <!-- 1. when the xref I encounter equal the path to parent file, do internal linking to top of page -->
        <!-- 28/08/2007 add fragment internal linking -->
        <!-- <xsl:when test="$include-xrefs/document/path[.=current()/@href]"> -->
        <xsl:when test="starts-with(@href,'#psFrag')">
          <xsl:attribute name="internal-destination">
            <!-- <xsl:value-of select="if (@frag='default') then $include-xrefs/document/uriid[.=current()/@uriid]/@start else @frag" /> -->
            <xsl:value-of select="@frag" />
          </xsl:attribute>
        </xsl:when>
        <!-- 2. when document exists somewhere in the page, do internal linking -->
        <!-- 28/08/2007 add fragment internal linking -->
        <!-- <xsl:when test="$include-xrefs/document/uriid[.=current()/@uriid][@frag='default' or @frag=current()/@frag]"> -->
        <xsl:when test="//root[@uriid=current()/@uriid]">
          <xsl:attribute name="internal-destination">
            <xsl:value-of select="@uriid" /><xsl:text>-</xsl:text>
            <!-- <xsl:value-of select="if (@frag='default') then $include-xrefs/document/uriid[.=current()/@uriid]/@start else @frag" /> -->
            <xsl:value-of select="@frag" />
          </xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="external-destination">
            <xsl:value-of select="psf:to-xref-path($siteprefix, $stylegroupname, @href)" />
          </xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:choose>
        <xsl:when test=".=''"><xsl:value-of select="@title" /></xsl:when>
        <xsl:otherwise><xsl:apply-templates /></xsl:otherwise>
      </xsl:choose>
    </xsl:element>
  </xsl:template>
<!-- ============================== Br (newline) ================================== -->
<!-- Template for PageSeeder newline (e.g. br)

.. admonition:: xpath:br

   | &#160;<fo:blocks/>

--> 
  <xsl:template match="br">
    &#160;<fo:block/>
  </xsl:template>

<!-- ============================== Graphic ================================== -->
<!-- Template for PageSeeder body graphic

.. admonition:: xpath:body/graphic

   | <fo:block>
   |   ...
   | </fo:block>

--> 
  <xsl:template match="body/graphic">
  	<fo:block>
      <xsl:call-template name="inline-graphic"/>
  	</fo:block>
  </xsl:template>

  <!--
  Template for PageSeeder graphic
  overriden because we need to load images using the jsessionid
-->

  <xsl:template name="inline-graphic" match="graphic">
    <fo:inline>
      <xsl:sequence select="psf:style-properties(., local-name(.))"/>     
	    <fo:external-graphic>
	      <xsl:attribute name="src">
	    <xsl:text>url(</xsl:text>
	      <xsl:choose>
	      <xsl:when test="starts-with(@src,'http://')">
	        <xsl:value-of select="@src" />
	      </xsl:when>
	      <xsl:when test="contains(@src,'..')">
	        <xsl:text>invalid</xsl:text>
	      </xsl:when>
	      <!-- uri id link -->
	      <xsl:when test="starts-with(@src, concat($siteprefix, '/uri/'))">
	        <xsl:text>http://</xsl:text>
	        <xsl:value-of select="$hosturl" />:<xsl:value-of select="$porturl" />
	        <xsl:value-of select="@src" />;jsessionid=<xsl:value-of select="$jsessionid" />
	      </xsl:when>
	      <!-- Temporary fix for graphics with incorrect source -->
	      <xsl:when test="not(starts-with(@src, concat($siteprefix, '/')))">
	        <xsl:text>http://</xsl:text>
	        <xsl:value-of select="$hosturl" />:<xsl:value-of select="$porturl" />
	        <xsl:value-of select="concat($siteprefix, @src)" />;jsessionid=<xsl:value-of select="$jsessionid" />
	      </xsl:when>
	      <xsl:otherwise>
	        <xsl:text>http://</xsl:text>
	        <xsl:value-of select="$hosturl" />:<xsl:value-of select="$porturl" />
	        <xsl:value-of select="@src" />;jsessionid=<xsl:value-of select="$jsessionid" />
	      </xsl:otherwise>
	      </xsl:choose>
	    <xsl:text>)</xsl:text>
	    </xsl:attribute>
	      <xsl:if test="@width">
	        <xsl:attribute name="content-width" select="concat(@width,'px')" />
	      </xsl:if>
	      <xsl:if test="@height">
	        <xsl:attribute name="content-height" select="concat(@height,'px')" />
	      </xsl:if>
	    </fo:external-graphic>
	  </fo:inline>
  </xsl:template>

<!-- ============================== PDF Bookmarks ================================== -->
    
  <xsl:template match="heading1"  mode="bookmark" />
  <xsl:template match="heading1[ancestor::section/@prefix[.='main']]"  mode="bookmark">
    <fo:bookmark internal-destination="{generate-id()}">
      <fo:bookmark-title><xsl:value-of select="."/></fo:bookmark-title>
      <xsl:for-each select="(following-sibling::heading2) |../following-sibling::body/heading2 | (../../following-sibling::section/body/heading2)">
          <xsl:apply-templates select="." mode="bookmark"/>
      </xsl:for-each>
    </fo:bookmark>
  </xsl:template>
  
  <xsl:template match="heading2[ancestor::section/@prefix[.='main' or .='front']]"  mode="bookmark">
    <fo:bookmark internal-destination="{generate-id()}">
      <fo:bookmark-title><xsl:value-of select="."/></fo:bookmark-title>
      <xsl:for-each select="(following-sibling::heading3) |../following-sibling::body/heading3 | (../../following-sibling::section/body/heading3)">
          <xsl:apply-templates select="." mode="bookmark"/>
      </xsl:for-each>
    </fo:bookmark>
  </xsl:template>
  
  <xsl:template match="heading3"  mode="bookmark">
    <fo:bookmark internal-destination="{generate-id()}">
      <fo:bookmark-title><xsl:value-of select="."/></fo:bookmark-title>
      <xsl:for-each select="(following-sibling::heading4) |../following-sibling::body/heading4 | (../../following-sibling::section/body/heading4)">
          <xsl:apply-templates select="." mode="bookmark"/>
      </xsl:for-each>
    </fo:bookmark>
  </xsl:template>
  
  <xsl:template match="heading4"  mode="bookmark">
    <fo:bookmark internal-destination="{generate-id()}">
      <fo:bookmark-title><xsl:value-of select="."/></fo:bookmark-title>
      <xsl:for-each select="(following-sibling::heading5) |../following-sibling::body/heading5 | (../../following-sibling::section/body/heading5)">
          <xsl:apply-templates select="." mode="bookmark"/>
      </xsl:for-each>
    </fo:bookmark>
  </xsl:template>

  <xsl:template match="heading5"  mode="bookmark">
    <fo:bookmark internal-destination="{generate-id()}">
      <fo:bookmark-title><xsl:value-of select="."/></fo:bookmark-title>
      <xsl:for-each select="(following-sibling::heading6) |../following-sibling::body/heading6 | (../../following-sibling::section/body/heading6)">
          <xsl:apply-templates select="." mode="bookmark"/>
      </xsl:for-each>
    </fo:bookmark>
  </xsl:template>
  
  <xsl:template match="heading6"  mode="bookmark">
    <fo:bookmark internal-destination="{generate-id()}">
      <fo:bookmark-title><xsl:value-of select="."/></fo:bookmark-title>
    </fo:bookmark>
  </xsl:template>
	
	<!-- Ignore ps:documentInfo, ps:documentinfo, ps:fragmentinfo element in PageSeeder document -->
	<xsl:template match="ps:documentInfo  | ps:documentinfo | ps:fragmentinfo"/>
</xsl:stylesheet>
