<?xml version="1.0"?>
<!--
  Templates used to display the files for a project.
  
  @author Jean-Baptiste Reure
  @author Christophe Lauret
  @version 18 August 2011
-->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:xs="http://www.w3.org/2001/XMLSchema"
                              xmlns:psf="http://www.pageseeder.com/function"
                              exclude-result-prefixes="xs psf">
  
  <!-- General Output properties (HTML5) -->
  <xsl:output method="html" encoding="utf-8" indent="no" undeclare-prefixes="no" media-type="text/html" />

  <xsl:variable name="context-path" select="/root/header/context-path"/>
  
  <!-- Main template -->
  <xsl:template match="root">
    <!-- Display the HTML Doctype -->
    <xsl:text disable-output-escaping="yes"><![CDATA[<!doctype html>
    ]]></xsl:text> 
    <html>
      <head>
        <title>Publish Engine Manager</title> 
        <meta http-equiv="Content-Language"    content="en"/>
        <meta http-equiv="Content-Script-Type" content="text/javascript"/>
        <meta http-equiv="Content-Style-Type"  content="text/css"/>
        <link rel="stylesheet" type="text/css" href="{$context-path}/css/jqueryui/pssilver/jquery-ui-1.8.10.css" media="screen"/>
        <link rel="stylesheet" type="text/css" href="{$context-path}/css/global.css" media="screen"/>
        <xsl:if test="unparsed-text-available(concat('../../../css/service/', header/service ,'.css'))">
          <link rel="stylesheet" type="text/css" href="{$context-path}/css/service/{header/service}.css" media="screen"/>
        </xsl:if>
        <script src="{$context-path}/js/lib/jquery/jquery-1.6.2.min.js"></script>
        <script src="{$context-path}/js/lib/jquery/jquery-ui-1.8.10.min.js"></script>
        <xsl:call-template name="ie-conditional-comments"/>
      </head>
      <body>
        <header id="berlioz-header" role="banner">
          <xsl:apply-templates select="." mode="header" />
        </header>
        <article>
          <div id="ps-content-{header/service}" class="content">
            <xsl:apply-templates select="content[@target = 'main']" />
          </div>
        </article>
        <!-- Auto load the script if it exists -->
        <xsl:if test="unparsed-text-available(concat('../../../js/service/', header/service ,'.js'))">
          <script src="{$context-path}/js/service/{header/service}.js"></script>
        </xsl:if>
      </body>
    </html>
  </xsl:template>
  
  <!-- Displays the header for the page -->
  <xsl:template match="root" mode="header">
    <div>
      <h1>Publisher Engine</h1>
      <xsl:if test="content[@name = 'user']/user">
        <xsl:variable name="s" select="header/service"/>
        <nav><ul>
          <li class="link{if ($s = 'projects' or $s = 'project') then ' active' else''}"><a href="{$context-path}/admin/projects.html">Projects</a></li>
          <li class="link{if ($s = 'status') then ' active' else''}"><a href="{$context-path}/admin/status.html">Status</a></li>
          <li class="link{if ($s = 'about') then ' active' else''}"><a href="{$context-path}/admin/about.html">About</a></li>
          <li class="link{if ($s = 'settings') then ' active' else''}"><a href="{$context-path}/admin/settings.html">Properties</a></li>
        </ul></nav>
      </xsl:if>
      <xsl:apply-templates select="content[@name = 'user']" />
    </div>
  </xsl:template>

  <xsl:template match="content[@name = 'user']">
    <div class="user">
      <xsl:choose>
        <xsl:when test="user">
          <span class="name">
            <xsl:value-of select="user/firstname" />
            <xsl:text>&#160;</xsl:text>
            <xsl:value-of select="user/surname" />
          </span>
          <xsl:text>&#160;|&#160;</xsl:text>
          <a href="{$context-path}/logout.html" class="logout">logout</a>
        </xsl:when>
        <!--<xsl:otherwise>
          <a href="#" class="login">login</a>
        </xsl:otherwise>
      --></xsl:choose>
    </div>
  </xsl:template>

  <!-- 
    Display the Scripts and Styles to be included in the HTML head
  -->
  <xsl:template name="ie-conditional-comments">
  <xsl:comment><![CDATA[[if IE 9]>
    <link rel="stylesheet" type="text/css" href="]]><xsl:value-of select="$context-path" /><![CDATA[/css/ie9.css"></link>
  <![endif]]]></xsl:comment>
  <xsl:comment><![CDATA[[if IE 8]>
    <link rel="stylesheet" type="text/css" href="]]><xsl:value-of select="$context-path" /><![CDATA[/css/ie8.css"></link>
    <script src="]]><xsl:value-of select="$context-path" /><![CDATA[/js/ie-html5.js" type="text/javascript"></script>
  <![endif]]]></xsl:comment>
  <xsl:comment><![CDATA[[if IE 7]>
    <link rel="stylesheet" type="text/css" href="]]><xsl:value-of select="$context-path" /><![CDATA[/css/ie7.css"></link>
    <script src="]]><xsl:value-of select="$context-path" /><![CDATA[/js/ie-html5.js" type="text/javascript"></script>
  <![endif]]]></xsl:comment>
  </xsl:template>
  
</xsl:stylesheet>
